@@ -1,6 +1,5 @@
 /*
-    Copyright (c) 2007-2011 iMatix Corporation
-    Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2013 Contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
 
@@ -25,7 +24,11 @@
 #include "../include/zmq.h"
 
 #include <assert.h>
+#if defined _WIN32_WCE
+#include "..\builds\msvc\errno.hpp"
+#else
 #include <errno.h>
+#endif
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -39,6 +42,11 @@
 #include <netdb.h>
 #endif
 
+// EPROTO is not used by OpenBSD and maybe other platforms.
+#ifndef EPROTO
+#define EPROTO 0
+#endif
+
 namespace zmq
 {
     const char *errno_to_string (int errno_);
@@ -52,7 +60,7 @@ namespace zmq
     const char *wsa_error ();
     const char *wsa_error_no (int no_);
     void win_error (char *buffer_, size_t buffer_size_);
-    void wsa_error_to_errno ();
+    int wsa_error_to_errno (int errcode);
 }
 
 //  Provides convenient way to check WSA-style errors on Windows.
