@@ -1,6 +1,5 @@
 /*
-    Copyright (c) 2007-2011 iMatix Corporation
-    Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2013 Contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
 
@@ -101,7 +100,7 @@ void zmq::io_object_t::out_event ()
     zmq_assert (false);
 }
 
-void zmq::io_object_t::timer_event (int id_)
+void zmq::io_object_t::timer_event (int)
 {
     zmq_assert (false);
 }
