@@ -1,6 +1,5 @@
 /*
-    Copyright (c) 2007-2011 iMatix Corporation
-    Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2013 Contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
 
@@ -27,28 +26,34 @@
 namespace zmq
 {
 
-    //  Class manages a set of outbound pipes. On send it load balances
+    //  This class manages a set of outbound pipes. On send it load balances
     //  messages fairly among the pipes.
-    class lb_t : public i_writer_events
+
+    class lb_t
     {
     public:
 
-        lb_t (class own_t *sink_);
+        lb_t ();
         ~lb_t ();
 
-        void attach (writer_t *pipe_);
-        void terminate ();
-        int send (zmq_msg_t *msg_, int flags_);
-        bool has_out ();
+        void attach (pipe_t *pipe_);
+        void activated (pipe_t *pipe_);
+        void pipe_terminated (pipe_t *pipe_);
 
-        //  i_writer_events interface implementation.
-        void activated (writer_t *pipe_);
-        void terminated (writer_t *pipe_);
+        int send (msg_t *msg_);
+
+        //  Sends a message and stores the pipe that was used in pipe_.
+        //  It is possible for this function to return success but keep pipe_
+        //  unset if the rest of a multipart message to a terminated pipe is
+        //  being dropped. For the first frame, this will never happen.
+        int sendpipe (msg_t *msg_, pipe_t **pipe_);
+
+        bool has_out ();
 
     private:
 
         //  List of outbound pipes.
-        typedef array_t <class writer_t> pipes_t;
+        typedef array_t <pipe_t, 2> pipes_t;
         pipes_t pipes;
 
         //  Number of active pipes. All the active pipes are located at the
@@ -64,12 +69,6 @@ namespace zmq
         //  True if we are dropping current message.
         bool dropping;
 
-        //  Object to send events to.
-        class own_t *sink;
-
-        //  If true, termination process is already underway.
-        bool terminating;
-
         lb_t (const lb_t&);
         const lb_t &operator = (const lb_t&);
     };
