@@ -1,8 +1,5 @@
 /*
-    Copyright (c) 2009-2011 250bpm s.r.o.
-    Copyright (c) 2007-2009 iMatix Corporation
-    Copyright (c) 2010-2011 Miru Limited
-    Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2013 Contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
 
@@ -37,7 +34,7 @@
 #include "io_object.hpp"
 #include "i_engine.hpp"
 #include "options.hpp"
-#include "decoder.hpp"
+#include "v1_decoder.hpp"
 #include "pgm_socket.hpp"
 
 namespace zmq
@@ -60,8 +57,9 @@ namespace zmq
         void plug (zmq::io_thread_t *io_thread_,
             zmq::session_base_t *session_);
         void terminate ();
-        void activate_in ();
-        void activate_out ();
+        void restart_input ();
+        void restart_output ();
+        void zap_msg_available () {}
 
         //  i_poll_events interface implementation.
         void in_event ();
@@ -72,6 +70,10 @@ namespace zmq
         //  Unplug the engine from the session.
         void unplug ();
 
+        //  Decode received data (inpos, insize) and forward decoded
+        //  messages to the session.
+        int process_input (v1_decoder_t *decoder);
+
         //  PGM is not able to move subscriptions upstream. Thus, drop all
         //  the pending subscriptions.
         void drop_subscriptions ();
@@ -88,7 +90,7 @@ namespace zmq
         struct peer_info_t
         {
             bool joined;
-            decoder_t *decoder;
+            v1_decoder_t *decoder;
         };
 
         struct tsi_comp
@@ -115,14 +117,13 @@ namespace zmq
         //  Associated session.
         zmq::session_base_t *session;
 
-        //  Most recently used decoder.
-        decoder_t *mru_decoder;
+        const pgm_tsi_t *active_tsi;
 
         //  Number of bytes not consumed by the decoder due to pipe overflow.
-        size_t pending_bytes;
+        size_t insize;
 
         //  Pointer to data still waiting to be processed by the decoder.
-        unsigned char *pending_ptr;
+        const unsigned char *inpos;
 
         //  Poll handle associated with PGM socket.
         handle_t socket_handle;
