@@ -1,6 +1,5 @@
 /*
-    Copyright (c) 2007-2011 iMatix Corporation
-    Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2013 Contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
 
@@ -27,9 +26,9 @@
 
 #ifdef ZMQ_HAVE_WINDOWS
 #include "windows.hpp"
+#define __PGM_WININT_H__
 #endif
 
-#define __PGM_WININT_H__
 #include <pgm/pgm.h>
 
 #ifdef ZMQ_HAVE_OSX
@@ -55,6 +54,9 @@ namespace zmq
 
         //  Initialize PGM network structures (GSI, GSRs).
         int init (bool udp_encapsulation_, const char *network_);
+
+        //  Resolve PGM socket address.
+        static int init_address(const char *network_, struct pgm_addrinfo_t **addr, uint16_t *port_number);
         
         //   Get receiver fds and store them into user allocated memory.
         void get_receiver_fds (fd_t *receive_fd_, fd_t *waiting_pipe_fd_);
