@@ -1,7 +1,5 @@
 /*
-    Copyright (c) 2009-2011 250bpm s.r.o.
-    Copyright (c) 2007-2009 iMatix Corporation
-    Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2013 Contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
 
@@ -116,6 +114,11 @@ void zmq::poll_t::stop ()
     stopping = true;
 }
 
+int zmq::poll_t::max_fds ()
+{
+    return -1;
+}
+
 void zmq::poll_t::loop ()
 {
     while (!stopping) {
