@@ -1,6 +1,5 @@
 /*
-    Copyright (c) 2007-2011 iMatix Corporation
-    Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2013 Contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
 
@@ -19,9 +18,10 @@
 */
 
 #include "pub.hpp"
+#include "msg.hpp"
 
-zmq::pub_t::pub_t (class ctx_t *parent_, uint32_t tid_) :
-    xpub_t (parent_, tid_)
+zmq::pub_t::pub_t (class ctx_t *parent_, uint32_t tid_, int sid_) :
+    xpub_t (parent_, tid_, sid_)
 {
     options.type = ZMQ_PUB;
 }
@@ -29,3 +29,15 @@ zmq::pub_t::pub_t (class ctx_t *parent_, uint32_t tid_) :
 zmq::pub_t::~pub_t ()
 {
 }
+
+int zmq::pub_t::xrecv (class msg_t *)
+{
+    //  Messages cannot be received from PUB socket.
+    errno = ENOTSUP;
+    return -1;
+}
+
+bool zmq::pub_t::xhas_in ()
+{
+    return false;
+}
