@@ -1,7 +1,5 @@
 /*
-    Copyright (c) 2007-2012 iMatix Corporation
-    Copyright (c) 2009-2011 250bpm s.r.o.
-    Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2013 Contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
 
@@ -40,7 +38,7 @@ namespace zmq
         ~pub_t ();
 
         //  Implementations of virtual functions from socket_base_t.
-        int xrecv (zmq::msg_t *msg_, int flags_);
+        int xrecv (zmq::msg_t *msg_);
         bool xhas_in ();
 
     private:
@@ -49,21 +47,6 @@ namespace zmq
         const pub_t &operator = (const pub_t&);
     };
 
-    class pub_session_t : public xpub_session_t
-    {
-    public:
-
-        pub_session_t (zmq::io_thread_t *io_thread_, bool connect_,
-            zmq::socket_base_t *socket_, const options_t &options_,
-            const address_t *addr_);
-        ~pub_session_t ();
-
-    private:
-
-        pub_session_t (const pub_session_t&);
-        const pub_session_t &operator = (const pub_session_t&);
-    };
-
 }
 
 #endif
