@@ -1,6 +1,5 @@
 /*
-    Copyright (c) 2007-2011 iMatix Corporation
-    Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2013 Contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
 
@@ -18,13 +17,12 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "../include/zmq.h"
-
 #include "rep.hpp"
 #include "err.hpp"
+#include "msg.hpp"
 
-zmq::rep_t::rep_t (class ctx_t *parent_, uint32_t tid_) :
-    xrep_t (parent_, tid_),
+zmq::rep_t::rep_t (class ctx_t *parent_, uint32_t tid_, int sid_) :
+    router_t (parent_, tid_, sid_),
     sending_reply (false),
     request_begins (true)
 {
@@ -35,7 +33,7 @@ zmq::rep_t::~rep_t ()
 {
 }
 
-int zmq::rep_t::xsend (zmq_msg_t *msg_, int flags_)
+int zmq::rep_t::xsend (msg_t *msg_)
 {
     //  If we are in the middle of receiving a request, we cannot send reply.
     if (!sending_reply) {
@@ -43,10 +41,10 @@ int zmq::rep_t::xsend (zmq_msg_t *msg_, int flags_)
         return -1;
     }
 
-    bool more = (msg_->flags & ZMQ_MSG_MORE);
+    bool more = msg_->flags () & msg_t::more ? true : false;
 
     //  Push message to the reply pipe.
-    int rc = xrep_t::xsend (msg_, flags_);
+    int rc = router_t::xsend (msg_);
     if (rc != 0)
         return rc;
 
@@ -57,7 +55,7 @@ int zmq::rep_t::xsend (zmq_msg_t *msg_, int flags_)
     return 0;
 }
 
-int zmq::rep_t::xrecv (zmq_msg_t *msg_, int flags_)
+int zmq::rep_t::xrecv (msg_t *msg_)
 {
     //  If we are in middle of sending a reply, we cannot receive next request.
     if (sending_reply) {
@@ -65,47 +63,42 @@ int zmq::rep_t::xrecv (zmq_msg_t *msg_, int flags_)
         return -1;
     }
 
+    //  First thing to do when receiving a request is to copy all the labels
+    //  to the reply pipe.
     if (request_begins) {
-
-        //  Copy the backtrace stack to the reply pipe.
-        bool bottom = false;
-        while (!bottom) {
-
-            //  TODO: What if request can be read but reply pipe is not
-            //  ready for writing?
-
-            //  Get next part of the backtrace stack.
-            int rc = xrep_t::xrecv (msg_, flags_);
+        while (true) {
+            int rc = router_t::xrecv (msg_);
             if (rc != 0)
                 return rc;
 
-            if ((msg_->flags & ZMQ_MSG_MORE)) {
+            if ((msg_->flags () & msg_t::more)) {
                 //  Empty message part delimits the traceback stack.
-                bottom = (zmq_msg_size (msg_) == 0);
+                bool bottom = (msg_->size () == 0);
 
                 //  Push it to the reply pipe.
-                rc = xrep_t::xsend (msg_, flags_);
-                zmq_assert (rc == 0);
+                rc = router_t::xsend (msg_);
+                errno_assert (rc == 0);
+
+                if (bottom)
+                    break;
             }
             else {
                 //  If the traceback stack is malformed, discard anything
                 //  already sent to pipe (we're at end of invalid message).
-                rc = xrep_t::rollback ();
-                zmq_assert (rc == 0);
+                rc = router_t::rollback ();
+                errno_assert (rc == 0);
             }
         }
-
         request_begins = false;
     }
 
-    //  Now the routing info is safely stored. Get the first part
-    //  of the message payload and exit.
-    int rc = xrep_t::xrecv (msg_, flags_);
+    //  Get next message part to return to the user.
+    int rc = router_t::xrecv (msg_);
     if (rc != 0)
-        return rc;
+       return rc;
 
     //  If whole request is read, flip the FSM to reply-sending state.
-    if (!(msg_->flags & ZMQ_MSG_MORE)) {
+    if (!(msg_->flags () & msg_t::more)) {
         sending_reply = true;
         request_begins = true;
     }
@@ -118,7 +111,7 @@ bool zmq::rep_t::xhas_in ()
     if (sending_reply)
         return false;
 
-    return xrep_t::xhas_in ();
+    return router_t::xhas_in ();
 }
 
 bool zmq::rep_t::xhas_out ()
@@ -126,6 +119,5 @@ bool zmq::rep_t::xhas_out ()
     if (!sending_reply)
         return false;
 
-    return xrep_t::xhas_out ();
+    return router_t::xhas_out ();
 }
-
