@@ -1,6 +1,5 @@
 /*
-    Copyright (c) 2007-2011 iMatix Corporation
-    Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2013 Contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
 
@@ -18,13 +17,12 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "select.hpp"
+#if defined ZMQ_USE_SELECT
+
 #include "platform.hpp"
-
-#include <string.h>
-#include <algorithm>
-
-#ifdef ZMQ_HAVE_WINDOWS
-#include "winsock2.h"
+#if defined ZMQ_HAVE_WINDOWS
+#include "windows.hpp"
 #elif defined ZMQ_HAVE_HPUX
 #include <sys/param.h>
 #include <sys/types.h>
@@ -36,7 +34,9 @@
 #include <sys/select.h>
 #endif
 
-#include "select.hpp"
+#include <string.h>
+#include <algorithm>
+
 #include "err.hpp"
 #include "config.hpp"
 #include "i_poll_events.hpp"
@@ -144,6 +144,11 @@ void zmq::select_t::stop ()
     stopping = true;
 }
 
+int zmq::select_t::max_fds ()
+{
+    return FD_SETSIZE;
+}
+
 void zmq::select_t::loop ()
 {
     while (!stopping) {
@@ -159,15 +164,17 @@ void zmq::select_t::loop ()
         //  Wait for events.
         struct timeval tv = {(long) (timeout / 1000),
             (long) (timeout % 1000 * 1000)};
-        int rc = select (maxfd + 1, &readfds, &writefds, &exceptfds,
-            timeout ? &tv : NULL);
-
 #ifdef ZMQ_HAVE_WINDOWS
+        int rc = select (0, &readfds, &writefds, &exceptfds,
+            timeout ? &tv : NULL);
         wsa_assert (rc != SOCKET_ERROR);
 #else
-        if (rc == -1 && errno == EINTR)
+        int rc = select (maxfd + 1, &readfds, &writefds, &exceptfds,
+            timeout ? &tv : NULL);
+        if (rc == -1) {
+            errno_assert (errno == EINTR);
             continue;
-        errno_assert (rc != -1);
+        }
 #endif
 
         //  If there are no events (i.e. it's a timeout) there's no point
@@ -209,3 +216,4 @@ bool zmq::select_t::is_retired_fd (const fd_entry_t &entry)
     return (entry.fd == retired_fd);
 }
 
+#endif
