@@ -1,6 +1,5 @@
 /*
-    Copyright (c) 2007-2011 iMatix Corporation
-    Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2013 Contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
 
@@ -22,48 +21,65 @@
 #define __ZMQ_TCP_LISTENER_HPP_INCLUDED__
 
 #include "fd.hpp"
-#include "ip.hpp"
+#include "own.hpp"
+#include "stdint.hpp"
+#include "io_object.hpp"
+#include "tcp_address.hpp"
+#include "../include/zmq.h"
 
 namespace zmq
 {
 
-    //  The class encapsulating simple TCP listening socket.
+    class io_thread_t;
+    class socket_base_t;
 
-    class tcp_listener_t
+    class tcp_listener_t : public own_t, public io_object_t
     {
     public:
 
-        tcp_listener_t ();
+        tcp_listener_t (zmq::io_thread_t *io_thread_,
+            zmq::socket_base_t *socket_, const options_t &options_);
         ~tcp_listener_t ();
 
-        //  Start listening on the interface.
-        int set_address (const char *protocol_, const char *addr_,
-            int backlog_);
+        //  Set address to listen on.
+        int set_address (const char *addr_);
 
-        //  Close the listening socket.
-        int close ();
-
-        //  Get the file descriptor to poll on to get notified about
-        //  newly created connections.
-        fd_t get_fd ();
-
-        //  Accept the new connection. Returns the file descriptor of the
-        //  newly created connection. The function may return retired_fd
-        //  if the connection was dropped while waiting in the listen backlog.
-        fd_t accept ();
+        // Get the bound address for use with wildcard
+        int get_address (std::string &addr_);
 
     private:
 
-        //  Address to listen on.
-        sockaddr_storage addr;
-        socklen_t addr_len;
+        //  Handlers for incoming commands.
+        void process_plug ();
+        void process_term (int linger_);
 
-        //  True, if the undelying file for UNIX domain socket exists.
-        bool has_file;
+        //  Handlers for I/O events.
+        void in_event ();
+
+        //  Close the listening socket.
+        void close ();
+
+        //  Accept the new connection. Returns the file descriptor of the
+        //  newly created connection. The function may return retired_fd
+        //  if the connection was dropped while waiting in the listen backlog
+        //  or was denied because of accept filters.
+        fd_t accept ();
+
+        //  Address to listen on.
+        tcp_address_t address;
 
         //  Underlying socket.
         fd_t s;
 
+        //  Handle corresponding to the listening socket.
+        handle_t handle;
+
+        //  Socket the listerner belongs to.
+        zmq::socket_base_t *socket;
+
+       // String representation of endpoint to bind to
+        std::string endpoint;
+
         tcp_listener_t (const tcp_listener_t&);
         const tcp_listener_t &operator = (const tcp_listener_t&);
     };
